@@ -74,11 +74,31 @@ class Puppet::Application::Cert < Puppet::Application
     options[:allow_dns_alt_names] = value
   end
 
+  option("--[no-]allow-authorization-extensions") do |value|
+    options[:allow_authorization_extensions] = value
+  end
+
   option("--verbose", "-v") do |arg|
     options[:verbose] = true
     set_log_level
   end
 
+  option("--human-readable", "-H") do |arg|
+    options[:format] = :human
+  end
+
+  option("--machine-readable", "-m") do |arg|
+    options[:format] = :machine
+  end
+
+  option("--interactive", "-i") do |arg|
+    options[:interactive] = true
+  end
+
+  option("--assume-yes", "-y") do |arg|
+    options[:yes] = true
+  end
+
   def help
     <<-'HELP'
 
@@ -132,7 +152,10 @@ The most important actions for day-to-day use are 'list' and 'sign'.
   List outstanding certificate requests. If '--all' is specified, signed
   certificates are also listed, prefixed by '+', and revoked or invalid
   certificates are prefixed by '-' (the verification outcome is printed
-  in parenthesis).
+  in parenthesis). If '--human-readable' or '-H' is specified,
+  certificates are formatted in a way to improve human scan-ability. If
+  '--machine-readable' or '-m' is specified, output is formatted concisely
+  for consumption by a script.
 
 * print:
   Print the full-text version of a host's certificate.
@@ -145,7 +168,10 @@ The most important actions for day-to-day use are 'list' and 'sign'.
   needs to be restarted after revoking certificates.
 
 * sign:
-  Sign an outstanding certificate request.
+  Sign an outstanding certificate request. If '--interactive' or '-i' is
+  supplied the user will be prompted to confirm that they are signing the
+  correct certificate (recommended). If '--assume-yes' or '-y' is supplied
+  the interactive prompt will assume the answer of 'yes'.
 
 * verify:
   Verify the named certificate against the local CA certificate.
@@ -187,6 +213,11 @@ configuration options can also be generated by running puppet cert with
   command line. The output of 'puppet cert list' shows any requested alt names
   for pending certificate requests.
 
+* --allow-authorization-extensions:
+  Enable the signing of a request with authorization extensions. Such requests
+  are sensitive because they can be used to write access rules in Puppet Server.
+  Currently, this is the only means by which such requests can be signed.
+
 * --digest:
   Set the digest for fingerprinting (defaults to the digest used when
   signing the cert). Valid values depends on your openssl and openssl ruby
@@ -246,6 +277,7 @@ Copyright (c) 2011 Puppet Labs, LLC Licensed under the Apache 2.0 License
     exit(Puppet.settings.print_configs ? 0 : 1) if Puppet.settings.print_configs?
 
     Puppet::SSL::Oids.register_puppet_oids
+    Puppet::SSL::Oids.load_custom_oid_file(Puppet[:trusted_oid_mapping_file])
 
     Puppet::Util::Log.newdestination :console
 
@@ -283,6 +315,7 @@ Copyright (c) 2011 Puppet Labs, LLC Licensed under the Apache 2.0 License
         exit
       end
     end
+
     result
   end
 
