@@ -118,7 +118,7 @@ parameter, so you can specify '--server <servername>' as an argument.
   code of '1' means at least one device had a compile failure, an exit code of
   '2' means at least one device had resource changes, and an exit code of '4'
   means at least one device had resource failures. Exit codes of '3', '5', '6',
-  or '7' means that a bitwise combination of the preceeding exit codes happened.
+  or '7' means that a bitwise combination of the preceding exit codes happened.
 
 * --help:
   Print this help message
@@ -199,7 +199,7 @@ Licensed under the Apache 2.0 License
 
         require 'puppet/configurer'
         configurer = Puppet::Configurer.new
-        configurer.run(:network_device => true, :pluginsync => Puppet[:pluginsync])
+        configurer.run(:network_device => true, :pluginsync => Puppet::Configurer.should_pluginsync?)
       rescue => detail
         Puppet.log_exception(detail)
         # If we rescued an error, then we return 1 as the exit code
