@@ -1,6 +1,11 @@
 module Puppet::Pops::Lookup
   class Invocation
-    attr_reader :scope, :override_values, :default_values, :explainer, :module_name
+    attr_reader :scope, :override_values, :default_values, :explainer
+    attr_accessor :module_name, :top_key
+
+    def self.current
+        nil # TODO, determine how to obtain the current lookup invocation.
+    end
 
     # Creates a context object for a lookup invocation. The object contains the current scope, overrides, and default
     # values and may optionally contain an {ExplanationAcceptor} instance that will receive book-keeping information
@@ -22,6 +27,7 @@ module Puppet::Pops::Lookup
       unless explainer.is_a?(Explainer)
         explainer = explainer == true ? Explainer.new : nil
       end
+      explainer = DebugExplainer.new(explainer) if Puppet[:debug] && !explainer.is_a?(DebugExplainer)
       @explainer = explainer
     end
 
@@ -43,6 +49,14 @@ module Puppet::Pops::Lookup
       end
     end
 
+    def emit_debug_info(preamble)
+      debug_explainer = @explainer
+      if debug_explainer.is_a?(DebugExplainer)
+        @explainer = debug_explainer.wrapped_explainer
+        debug_explainer.emit_debug_info(preamble)
+      end
+    end
+
     # The qualifier_type can be one of:
     # :global - qualifier is the data binding terminus name
     # :data_provider - qualifier a DataProvider instance
@@ -50,34 +64,31 @@ module Puppet::Pops::Lookup
     # :merge - qualifier is a MergeStrategy instance
     # :module - qualifier is the name of a module
     # :interpolation - qualifier is the unresolved interpolation expression
-    # :meta - qualifier is the name of a module or nil if that's not applicable
+    # :meta - qualifier is the module name
+    # :data - qualifier is the key
     #
     # @param qualifier [Object] A branch, a provider, or a path
     def with(qualifier_type, qualifier)
-      is_meta = qualifier_type == :meta
-      @module_name = qualifier if is_meta
       if @explainer.nil?
         yield
       else
-        if is_meta
-          save_explainer = @explainer
-          @explainer = nil
-          begin
-            yield
-          ensure
-            @explainer = save_explainer
-          end
-        else
-          @explainer.push(qualifier_type, qualifier)
-          begin
-            yield
-          ensure
-            @explainer.pop
-          end
+        @explainer.push(qualifier_type, qualifier)
+        begin
+          yield
+        ensure
+          @explainer.pop
         end
       end
     end
 
+    def only_explain_options?
+      @explainer.nil? ? false : @explainer.only_explain_options?
+    end
+
+    def explain_options?
+      @explainer.nil? ? false : @explainer.explain_options?
+    end
+
     def report_found_in_overrides(key, value)
       @explainer.accept_found_in_overrides(key, value) unless @explainer.nil?
       value
@@ -93,6 +104,10 @@ module Puppet::Pops::Lookup
       value
     end
 
+    def report_merge_source(merge_source)
+      @explainer.accept_merge_source(merge_source) unless @explainer.nil?
+    end
+
     # Report the result of a merge or fully resolved interpolated string
     # @param value [Object] The result to report
     # @return [Object] the given value
@@ -112,6 +127,12 @@ module Puppet::Pops::Lookup
     def report_module_not_found
       @explainer.accept_module_not_found unless @explainer.nil?
     end
+
+    def report_text(&block)
+      unless @explainer.nil?
+        @explainer.accept_text(block.call)
+      end
+    end
   end
 end
 
