@@ -134,12 +134,13 @@ class TypeParser
   # @api private
   def self.type_map
     @type_map ||= {
-       'integer'       => TypeFactory.integer,
-       'float'         => TypeFactory.float,
+        'integer'      => TypeFactory.integer,
+        'float'        => TypeFactory.float,
         'numeric'      => TypeFactory.numeric,
         'iterable'     => TypeFactory.iterable,
         'iterator'     => TypeFactory.iterator,
         'string'       => TypeFactory.string,
+        'binary'       => TypeFactory.binary,
         'sensitive'    => TypeFactory.sensitive,
         'enum'         => TypeFactory.enum,
         'boolean'      => TypeFactory.boolean,
@@ -167,10 +168,12 @@ class TypeParser
         'typealias'    => TypeFactory.type_alias,
         'typereference' => TypeFactory.type_reference,
         'typeset'      => TypeFactory.type_set,
-      # A generic callable as opposed to one that does not accept arguments
+         # A generic callable as opposed to one that does not accept arguments
         'callable'     => TypeFactory.all_callables,
         'semver'       => TypeFactory.sem_ver,
-        'semverrange'  => TypeFactory.sem_ver_range
+        'semverrange'  => TypeFactory.sem_ver_range,
+        'timestamp'    => TypeFactory.timestamp,
+        'timespan'     => TypeFactory.timespan
     }
   end
 
@@ -355,6 +358,9 @@ class TypeParser
 
     when 'callable'
       # 1..m parameters being types (last three optionally integer or literal default, and a callable)
+      if parameters.size > 1 && parameters[0].is_a?(Array)
+        raise_invalid_parameters_error('callable', '2 when first parameter is an array', parameters.size) unless parameters.size == 2
+      end
       TypeFactory.callable(*parameters)
 
     when 'struct'
@@ -478,6 +484,14 @@ class TypeParser
       raise_invalid_parameters_error('Runtime', '2', parameters.size) unless parameters.size == 2
       TypeFactory.runtime(*parameters)
 
+    when 'timespan'
+      raise_invalid_parameters_error('Timespan', '0 to 2', parameters.size) unless parameters.size <= 2
+      TypeFactory.timespan(*parameters)
+
+    when 'timestamp'
+      raise_invalid_parameters_error('Timestamp', '0 to 2', parameters.size) unless parameters.size <= 2
+      TypeFactory.timestamp(*parameters)
+
     when 'semver'
       raise_invalid_parameters_error('SemVer', '1 or more', parameters.size) unless parameters.size >= 1
       TypeFactory.sem_ver(*parameters)
